<?php
session_start();
// PJC 07 July 2020 Updated wording to SoftPBX
// PJC13SEP2018 - Updated ldappassword
// PJC12AUG2018 - Increased timeout, runtime and refresh to allow larger configs.
// PJC17FEB2015 - Layout
// PJC13MARCH14
// PJCNOV2013 PJCSEPT2013
//bobg 31jan07 2feb 22mar 13may 25aug 16sep 5oct 26jan08 22feb 27feb 12mar 13apr 23apr 19may 31jul 25jun09 10aug 14aug 21apr10 24feb11 3nov
// 14mar12
header('Content-type: text/html; charset=UTF-8') ;
$page_id = "068";
include "assist/.access.php";
include "assist/tzone.php"; // include timezone
$script=basename($_SERVER['SCRIPT_NAME']);

//print "USER_AGENT: [".$_SERVER['HTTP_USER_AGENT']."]";
//if (strpos($_SERVER['HTTP_USER_AGENT'],"MSIE")) $runtime=180; else $runtime=50; //if its taking to long give up before the browser does

$runtime=180;
$timeout = 5;

$ds = @ldap_connect("127.0.0.1",4000) or exit("<p>EXIT: LDAP connect failure</p>");
ldap_set_option($ds, LDAP_OPT_PROTOCOL_VERSION, 3); // this line required for ldap_rename() to work.
@ldap_bind($ds,"INTERNAL","1122334455") or exit("<p>EXIT: LDAP bind failure</p>");

$sr = ldap_read($ds,"cn=Manager,cn=System","objectclass=Administrator"); 
$info = ldap_get_entries($ds,$sr);
$ldappasssword = $info[0]['userpassword'][0];
ldap_close($ds);


//the following is just incase this page is used as a home page so we can remember user etc
// use home page of http://192.168.0.1/users.php?u=%u&e=%e&p=%p

if ( ! isset($_SESSION['user'])) {
   $_SESSION['user'] = $_SESSION['extn'] = $_SESSION['port'] = "";
}
if (isset ($_GET['u'])) {
   $_SESSION['user'] = $_GET['u'];
   $_SESSION['extn'] = $_GET['e'];
   $_SESSION['port'] = $_GET['p'];
}
if (isset($_GET['x'])) $_SESSION['refresh'] = 90000; else $_SESSION['refresh'] = 10000;
if (isset($_GET['x'])) $x=$_GET['x']; else $x=0;
if (isset($_GET['y'])) $y=$_GET['y']; else $y=0;
$_SESSION['level']=$_SESSION['auto1']=$_SESSION['auto2']=0;
?>
<html>
<head>
<title>User Location Database Check</title>
<META HTTP-EQUIV="Expires" CONTENT="0">
<META HTTP-EQUIV="Pragma" CONTENT="no-cache">
<META HTTP-EQUIV="Cache-Control" CONTENT="no-cache">
<META http-equiv="Content-Type" content="text/html; charset=UTF-8">
<link href="assist/css/Styles.css" rel="stylesheet" type="text/css">
<script src="assist/js/wiz/jquery-1.8.0.js" type="text/javascript"></script>
<script src="assist/js/wiz/main.js" type="text/javascript"></script>
<script type="text/javascript">
<!--
function getPage() {
   window.location.href="<?php echo $script?>?x="+document.body.scrollLeft+"&y="+document.body.scrollTop ;
}
var timerId = setTimeout("getPage()",<?php echo $_SESSION['refresh']?>);
//-->
</script>
<script type="text/javascript">
$(document).ready(function(){
	 $("#tbl").sticky();
});
$(function() {
	$(window).scroll(function() {
		if($(this).scrollTop() != 0) {
			$('#toTop').fadeIn();
		} else {
			$('#toTop').fadeOut();
		}
	});
	$('#toTop').click(function() {
		$('body,html').animate({scrollTop:0},800);
	});
});
</script>
<script language="javascript">
var popupWindow = null;
function centeredPopup(url,winName,w,h,scroll){
LeftPosition = (screen.width) ? (screen.width-w)/2 : 0;
TopPosition = (screen.height) ? (screen.height-h)/2 : 0;
settings =
'height='+h+',width='+w+',top='+TopPosition+',left='+LeftPosition+',scrollbars='+scroll+',resizable'
popupWindow = window.open(url,winName,settings)
}
</script>
</head>
<body onLoad="window.scrollTo(<?php echo $x*1?>,<?php echo $y*1?>)">
<div id="toTop">^ Back to Top</div>
<div style="position:absolute; top:8px; right:8px" >
<img onClick="window.location = '<?php echo $script?>'" src="assist/images/refresh16.gif" alt="refresh" title="refresh" border="0">&nbsp;&nbsp;
<img onClick="centeredPopup('assist/help/help_location.php','help','600','500','yes');return false" src="assist/images/help16.gif" alt="help" title="help" border="0">&nbsp;&nbsp;
<img onClick="window.location = 'assist/assist.php'" src="assist/images/assist16.gif" alt="manager assist" title="manager assist" border="0">&nbsp;&nbsp;
<img onClick="window.location = 'index.php'" src="assist/images/home16.gif" alt="home" title="home" border="0">
</div>
<div style="position:absolute; top:30px; right:8px" >
<a href="select_modules.php">Select Modules</a>&nbsp;|&nbsp;<a href="<?php echo $script?>?zap=1&x=0&y=0">Move unregistered users to Zero Location</a>
</div>
<div id="contain">
<h2><img src='assist/images/backup.png' style="vertical-align:middle;">&nbsp;User Location Database Check</h2>
<p><br/>Now checking that User locations are consistent across all active Call Servers...</p>
<?php
sleep(2);

if (isset($_GET['toggle'])) {
   if ($_SESSION['modules2']['disable'][$_GET['toggle']] == 1) {
      $_SESSION['modules2']['disable'][$_GET['toggle']]=0;
      $status="enabled";
   } else {
      $_SESSION['modules2']['disable'][$_GET['toggle']]=1;
      $status="disabled";
   }
   print "Diagnostic tests on ".$_SESSION['modules']['cn'][$_GET['toggle']]." are $status.";
}
if (isset($_GET['kick'])) {
   $_SESSION['kicklocn']=$_SESSION['modules']['guid'][$_GET['kick']];
   print "Kick location now: ".$_SESSION['modules']['cn'][$_GET['kick']]."<br/>";
}
if (!isset($_SESSION['kicklocn'])) $_SESSION['kicklocn'] = '00000000-0000-0000-0000-000000000000';

if (isset($_GET['test'])) {
   $test=($_GET['test']);
   print "test - User: zczc test, Call Server: ".$_SESSION['modules']['cn'][$test].", Action: ";
   $ds = @ldap_connect($_SESSION['modules']['ipaddress0'][$test],4000) or exit("<p>EXIT: LDAP connect failure</p>");
   @ldap_bind($ds,"Manager",$ldappasssword) or exit("<p>EXIT: LDAP bind failure</p>");
   $sr=@ldap_read($ds,"cn=zczc test,cn=Users","objectclass=User", array());
   $info=@ldap_get_entries($ds,$sr);
   if ($info['count']==0) {
      if ($_SESSION['modules']['ipaddress'][$test]!="127.0.0.1") {
         print "added.";
         $att=array('objectclass'=>'User','telephonenumber'=>'0*0','disabled'=>'1','description'=>'system test user - delete at will','location'=>$_SESSION['modules']['guid'][$test]);
         ldap_add($ds,"cn=zczc test,cn=Users",$att);
         $_SESSION['doblankname']=1;
         print " Enable blank name check. ";
      }
   } else {
      if ($_SESSION['modules']['ipaddress'][$test]!="127.0.0.1") {
         print "moved.";
         $att=array('location'=>$_SESSION['modules']['guid'][$test]);
         ldap_modify($ds,"cn=zczc test,cn=Users",$att);
      } else {
         print "deleted." ;
         ldap_delete($ds,"cn=zczc test,cn=Users");
         @ldap_delete($ds,"cn=ZCZC TEST,cn=Users");
      }
   }
   if ($_SESSION['modules']['ipaddress'][$test]!="127.0.0.1") {
      // modules sometimes fail to pass on their version number on upgrade -e.g. double reboots, so check
      $sr=ldap_read($ds,"guid=".$_SESSION['modules']['guid'][$test],"objectclass=*");
      $info=@ldap_get_entries($ds,$sr);
      $v = $info[0]['productversion'][0];
      if ($_SESSION['modules']['version'][$test]!= $v) {
         $att=array('productversion'=>$v."?");
         ldap_modify($ds,"guid=".$_SESSION['modules']['guid'][$test],$att);
         $att=array('productversion'=>$v);
         ldap_modify($ds,"guid=".$_SESSION['modules']['guid'][$test],$att);
         print " Module version > $v ";
      }
      else print " Module version OK. ";
   }
   ldap_close($ds);
}
if(!isset($_GET['x'])) {
   print "</body></html>";
   exit();
}
$start =time();
$lost = ""; //store last telempne not registered on all nodes
if (isset($_GET['zap'])) $zap=1; else $zap=0;
$_SESSION['level']=0;
if ($_SESSION['level']==0) {

if (!isset($_SESSION['selected']))  $_SESSION['modules']=array();
   $ds = @ldap_connect("127.0.0.1",4000) or exit("<p>EXIT: LDAP connect failure</p>");
   ldap_set_option($ds, LDAP_OPT_PROTOCOL_VERSION, 3); // this line required for ldap_rename() to work.
   @ldap_bind($ds,"Manager",$ldappasssword) or exit("<p>EXIT: LDAP bind failure</p>");
   if ($ds) {
      if (isset($_SESSION['doblankname'])) {
         unset($_SESSION['doblankname']);
         // ldap_list does not find entries with blank names - so expose them
         $att=array('cn','guid');
         $sr=ldap_search($ds,"*", "(&(objectclass=*)(cn=))",$att);
         $info=ldap_get_entries($ds,$sr);
         if( $info['count']){
            //print $info['count']." database entries found without names: <br/>";
            for($n=0; $n<$info['count'];$n++) {
               $dn=$info[$n]['dn'];
               if(strpos($dn,"cn=Connections")) continue;
               $err="aarghhh - this name was missing and hiding entry - ".substr($info[$n]['guid'][0],0,8);
               $dn2="cn=$err";
               $pa=explode(',',$dn);
               unset($pa[0]);
               $parent=implode(',',$pa);
               print "<b>$err,$parent</b><br/>";
               ldap_rename($ds,$dn,$dn2,$parent,true);
               $fpw=fopen("/WebManager/warnings.txt","a");
               $w = date("d/m/Y H:i ",time())."$script - $err,$parent\n";
               fputs($fpw,$w);
               fclose($fpw);
            }
         } else print "No blank names found - good.<br/>";
      }

if (!isset($_SESSION['selected'])) {
      // find self
      $sr=ldap_read($ds,"cn=Self,cn=Modules","objectclass=*");
      $info=ldap_get_entries($ds,$sr);
      $self = $info[0]['location'][0];
      // find master and put at start of array
      $sr=@ldap_search($ds,"cn=Modules","(&(objectclass=AdministrationModule)(administrationmode=1))");
		$info=ldap_get_entries($ds,$sr);
      $sa = explode(",",$info[0]['dn']);
	   $s = substr($sa[1],3);
	   $_SESSION['modules']['cn'][0] = $s;
	   $_SESSION['modules']['guid'][0]="";
      $_SESSION['modules']['lastseen'][]=0;
	   $_SESSION['modules']['ipaddress'][0]="";
      $_SESSION['modules']['ipaddress0'][0]="";
      $_SESSION['modules']['version'][0]="";
      $_SESSION['modules']['type'][0]="";
      // build array of nodes
      // Changed by CNH to allow for 5500 NSG 19 April 2013
	   $sr=@ldap_list($ds,"cn=Modules","(&(objectclass=Module)(modulestatus=0)(|(moduletype=1)(moduletype=10)(moduletype=11)(moduletype=12)(moduletype=17)(moduletype=18)(moduletype=19)))");
	   $info=ldap_get_entries($ds,$sr);
//print "Module count = ".$info['count']."<br/>";
	   for($n=0; $n<$info['count'];$n++) {
		   if ($info[$n]['cn'][0] == $_SESSION['modules']['cn'][0]) $n2 = 0;
		   else $n2 = count($_SESSION['modules']['cn']);
		   $_SESSION['modules']['cn'][$n2] = $info[$n]['cn'][0];
		   $_SESSION['modules']['guid'][$n2] = $info[$n]['guid'][0];
         $_SESSION['modules']['version'][$n2] = $info[$n]['productversion'][0];
         $_SESSION['modules']['lastseen'][$n2] = floor((time(0)-$info[$n]['lastseen'][0])/(60*60*24));
         if ($info[$n]['moduletype'][0]==1) $_SESSION['modules']['type'][$n2]="4100";
         if ($info[$n]['moduletype'][0]==1 and substr($info[$n]['serialnumber'][0],0,8)!="00-07-d9") $_SESSION['modules']['type'][$n2]="XS";
         if ($info[$n]['moduletype'][0]==10) $_SESSION['modules']['type'][$n2]="4140";
         if ($info[$n]['moduletype'][0]==11) $_SESSION['modules']['type'][$n2]="5100";
         if ($info[$n]['moduletype'][0]==12) $_SESSION['modules']['type'][$n2]="5108";
         if ($info[$n]['moduletype'][0]==17) $_SESSION['modules']['type'][$n2]="SoftPBX Linux";
         if ($info[$n]['moduletype'][0]==18) $_SESSION['modules']['type'][$n2]="SoftPBX Apple";
// Added by CNH to cope with 5500 NSG 19 April 2013
         if ($info[$n]['moduletype'][0]==19) $_SESSION['modules']['type'][$n2]="5500 (NSG)";
   	   $_SESSION['modules']['ipaddress'][$n2] = $info[$n]['ipaddress'][0];
         $_SESSION['modules']['ipaddress0'][$n2] = $info[$n]['ipaddress'][0];
         $dn = "cn=1,".$info[$n]['dn'];
         $sr=ldap_read($ds,$dn,"objectclass=*");
         $infox=ldap_get_entries($ds,$sr);
         $_SESSION['modules']['publicipaddress'][$n2] = $infox[0]['publicipaddress'][0];
         if ($infox[0]['publicipaddress'][0]!="") $_SESSION['modules']['ipaddress0'][$n2] = $infox[0]['publicipaddress'][0];

         $dn="guid=$self";
         $search="(&(objectclass=Member)(member=".$info[$n]['guid'][0]."))";
         $sr=ldap_search($ds,$dn,$search);
         $info_m=ldap_get_entries($ds,$sr);
         if ($info_m['count']) {
 // print "found one".$info[$n]['cn'][0];
            $dn=substr($info_m[0]['dn'],strpos($info_m[0]['dn'],',')+1);
            $sr=ldap_read($ds,$dn,"objectclass=*");
            $info_imt=ldap_get_entries($ds,$sr);
            if ($info_imt[0]['ipaddress'][0]!="") $_SESSION['modules']['ipaddress0'][$n2]=$info_imt[0]['ipaddress'][0];
         }
         if ($info[$n]['guid'][0]==$self) $_SESSION['modules']['ipaddress0'][$n2]="127.0.0.1";
 //print $info[$n]['cn'][0]." ";
      }
		ldap_close($ds);
      $_SESSION['modules']['cn'][] = "Zero Location";
		$_SESSION['modules']['guid'][]='00000000-0000-0000-0000-000000000000';
      $_SESSION['modules']['lastseen'][]=0;
		$_SESSION['modules']['ipaddress'][]="127.0.0.1";
      $_SESSION['modules']['publicipaddress'][]="&nbsp";
      $_SESSION['modules']['ipaddress0'][]="127.0.0.1";
      $_SESSION['modules']['version'][]="&nbsp";
      $_SESSION['modules']['type'][]="&nbsp";
//		array_multisort($_SESSION['modules']['cn'], $_SESSION['modules']['guid'], $_SESSION['modules']['ipaddress'], SORT_ASC, SORT_STRING);
//print_r ($_SESSION['modules']);
} // end selected



      $run=time()-$start;
      print "<br/>($run Seconds so far to build this page)<br/>";

      $z=0;
      print "<table border=1 width='100%' id='tbl'>";
      	print "<tr><th>Number&nbsp;<a href='#' class='tooltip' title='\"Number\" - List position. Position 0 is always the primary/master.<br/>Clicking on this will disable/enable diagnostics on that Call Server<br/>Useful when a (faulty?) unit is blocking the view of the rest of the system.'><img src='assist/images/tooltip.gif'></a></th>
      	<th width='50px'>Kick&nbsp;<a href='#' class='tooltip' title='\"Kick Here\" - Lost users are normally kicked to Zero Location, but that can be changed.'><img src='assist/images/tooltip.gif'></a></th>
      	<th width='50px'>Test&nbsp;<a href='#' class='tooltip' title='\"test\" - This will move a disabled User \"zczc test\" to that Call Server, by a command on that Call Server.<br/>Thus check if that Call Server can communicate with the primary/master. Moving the test user to Zero Location will delete it.<br/>Checks module version number. Checks and corrects blank names when test user added. <br/>Blank entries will now have name of \"aarghhh - this name was missing and hiding entry\"'><img src='assist/images/tooltip.gif'></a></th>
      	<th width='100px'>RealIP&nbsp;<a href='#' class='tooltip' title='\"Real IP address\" - e.g. 192.168.0.1'><img src='assist/images/tooltip.gif'></a></th>
      	<th width='100px'>PublicIP&nbsp;<a href='#' class='tooltip' title='\"Public IP address\" - e.g. 80.79.78.76'><img src='assist/images/tooltip.gif'></a></th>
      	<th width='100px'>ConnectIP&nbsp;<a href='#' class='tooltip' title='\"Connecting using IP address\" - The IP address the call server you are running this web page on will use as the destination address for the particular Call Server.<br/> May be it&apos;s public IP address or it&apos;s real one.'><img src='assist/images/tooltip.gif'></a></th>
      	<th width='120px'>ModuleName&nbsp;<a href='#' class='tooltip' title='\"Module Name\" - As seen by Manager'><img src='assist/images/tooltip.gif'></a></th>
      	<th width='120px'>ModuleType&nbsp;<a href='#' class='tooltip' title='\"Module Type\" - e.g. 4100, 4140, 5100, 5108, SoftPBX'><img src='assist/images/tooltip.gif'></a></th>
      	<th width='100px'>Ver&nbsp;<a href='#' class='tooltip' title='\"Module Firmware Version\" - e.g. 3.2(1625)'><img src='assist/images/tooltip.gif'></a></th>
      	<th width='100px'>LastSeen&nbsp;<a href='#' class='tooltip' title='\"Last Seen\" - Number of days since database update from this call server.<br/>Typically 0, 2 or greater requires explaining.'><img src='assist/images/tooltip.gif'></a></th>
      	<th>State&nbsp;<a href='#' class='tooltip' title='\"Current State\" - Shows number of users as seen by this module and includes time taken.<br/>Error is shown if unable to communicate with module.'><img src='assist/images/tooltip.gif'></a></th>
      	</tr>";

		foreach ($_SESSION['modules']['ipaddress0'] as $key => $value) {
		   if ($_SESSION['modules']['guid'][$key]==$_SESSION['kicklocn'])$kh=1; else $kh=0;
		   print "<tr><td><a title='disable/enable' href=\"".$script."?toggle=".$key."\">".$key."</a></td>
            <td><a title='kick here' href=\"".$script."?kick=".$key."\">$kh</a></td>
            <td><a title='move zczc test user here' href=\"".$script."?test=".$key."\">test</a></td>
            <td><a title='real ip address'>".$_SESSION['modules']['ipaddress'][$key]."</a></td>
            <td><a title='public ip address'>".$_SESSION['modules']['publicipaddress'][$key]."</a></td>
            <td><a title='connecting using ip address'>".$_SESSION['modules']['ipaddress0'][$key]."</a></td>
            <td>".$_SESSION['modules']['cn'][$key]."</td>
            <td>".$_SESSION['modules']['type'][$key]."</td><td>".$_SESSION['modules']['version'][$key]."</td>
            <td><a title='last seen ".$_SESSION['modules']['lastseen'][$key]." days ago'>".$_SESSION['modules']['lastseen'][$key]."</a></td>
            <td>";
         if (!isset($_SESSION['modules2']['disable'][$key]) or $_SESSION['modules2']['disable'][$key]!=1) {
            $fp = @fsockopen("tcp://$value", 4000, $errno, $errstr, $timeout);
		      if (!$fp) {
		         echo "<b>ERROR:</b> $errno - $errstr ";
               $ds=false;
		      } else {
		         fclose($fp);
		         $ds = ldap_connect($value,4000);
            }
			   if ( ! $ds) echo "LDAP connect failure";
			   else {
			      $r = @ldap_bind($ds,"Manager",$ldappasssword);
				   if ( ! $r) {
					   echo "LDAP bind failure";
                  $ds=false;
				   }
            }
         } else {
            print "<b>Disabled</b>";
            $ds=false;
         }
			if ($ds) {
				$_SESSION['modules']['live'][$key]=1;
				$attributes=array("location","cn");
				$sr=ldap_list($ds,"cn=Users","objectclass=user",$attributes);
			   $info[$key]=ldap_get_entries($ds,$sr);
            if ($info[$key]['count']!= $info[0]['count']) {
               $_SESSION['modules']['live'][$key]=0;
               print "<b>WARNING:</b> "; $z++;
            }
		      print "users=".$info[$key]['count'];

      //get some licence info from master for later
            if ($key==0) {
               $ipuser=$ipuserused=$ipvirtualuser=$ipvirtualuserused=$pcsuser=$pcsuserused=0;
               $sr=@ldap_list($ds,"cn=System","objectclass=licenserecord");
               $info2=ldap_get_entries($ds,$sr);
               for($n=0; $n<$info2['count'];$n++) {
                  if ($info2[$n]['license'][0]=="_IPUser") {
                     $ipuser = $info2[$n]['qty'][0];
                     $ipuserused = $info2[$n]['used'][0];
                  }
                  if ($info2[$n]['license'][0]=="_IPVirtualUser") {
                     $ipvirtualuser = $info2[$n]['qty'][0];
                     $ipvirtualuserused = $info2[$n]['used'][0];
                  }
                  if ($info2[$n]['license'][0]=="_PCSUser") {
                     $pcsuser = $info2[$n]['qty'][0];
                     $pcsuserused = $info2[$n]['used'][0];
                  }
                  if ($info2[$n]['license'][0]=="_POTS") {
                     $pots = $info2[$n]['qty'][0];
                     $potsused = $info2[$n]['used'][0];
                  }
               }
            }
            ldap_close($ds);

	      } else $_SESSION['modules']['live'][$key]=0;
         $run=time()-$start;
         print "&nbsp($run Seconds)</td></tr>";
         if ($run>$runtime) {
            print "</table><table><p><b>WARNING: </b>TAKING TOO LONG - CUTTING SHORT <b>:WARNING</b></p>";
            break;
         }
      }
      print "</table>";
      if (!$_SESSION['modules']['live'][0]) {
         print("<p>Cannot contact Primary Call Server - Abandoning</p>");
      }
      else $_SESSION['level'] = 1;

      if ($_SESSION['level']) {
         // display licence count - collected above
         print "<p><b>Licence count.</b> Note: Adding a new/duff licence will trigger a recount.<br/>";
         if($ipuser>0) print "IPUser licences: $ipuser available $ipuserused used. ";
         if($ipvirtualuser>0)print "IPVirtualUser licences: $ipvirtualuser available $ipvirtualuserused used. ";
         if($pcsuser>0) print "PCSUser licences: $pcsuser added to IPUser. ";
         if($pots>0) print "POTS licences: $pots available $potsused used. ";
         print "</p>";

         // check user name on each module against master
         print "<p>Checking module user list against master.</p>";
   	   for($n=0; $n<$info[0]['count'];$n++) {
      	   $z=0;
	  	   	foreach ($_SESSION['modules']['live'] as $key => $value) {
               if ($value==0) continue;
	   		   if ($key>0) {
//print "($key)";  // module number
                  if ($info[0][$n]['cn'][0] != $info[$key][$n]['cn'][0]) {
			   	      $z++;
                     print $_SESSION['modules']['cn'][$key]." ";
                     //      $_SESSION['modules']['live'][$key]=0;
   	   			}
	  		   	}
	   	   }
		      if ($z>0){
			      print " has lost user step at ".$info[0][$n]['cn'][0]."<br/>";
  		      }
         }

         // check user location on each module against master
         $la = array();
         if (!isset($_SESSION['lost2'])) $_SESSION['lost2']=array();
         if (!isset($_SESSION['lost3'])) $_SESSION['lost3']=array();
         for($n=0; $n<$info[0]['count'];$n++) {
   	      $z=0;
            foreach ($_SESSION['modules']['live'] as $key => $value) {
               if ($value==0) continue;
      			if ($key>0) {
//print "($key)";  // module number
                  if ($info[0][$n]['location'][0] != $info[$key][$n]['location'][0]) {
		   		      $z++;
                     print $_SESSION['modules']['cn'][$key].", ";
				      }
               }
            }
	   	   if ($z>0){
		   	   print " has lost ".$info[0][$n]['cn'][0]."<br/>";
			      $lost = $la[] = $info[0][$n]['cn'][0];
   		   }
         }
         if ($lost == "") {
            print "<p>All visible Call Servers agree.</p>";
            $_SESSION['level']=100;
         } else {
            foreach($la as $key => $value) {
               if( in_array($value,$_SESSION['lost2']) and in_array($value,$_SESSION['lost3'])) $la2[]=$value;
            }
            if (count($la2)>0) {
               $lost = $la2[rand(0,(count($la2)-1))];
               unset($la[array_search($lost,$la)]); //clear is so we got to count to 3 again
	   	      $ds = ldap_connect("127.0.0.1",4000);
  		   	   if ( ! $ds) echo "LDAP connect failure<br>";
               else {
	   	         $r = @ldap_bind($ds,"Manager",$ldappasssword);
		   	      if ( ! $r) {
			            echo "LDAP bind failure<br>";
     				   }
      		   	$sr=@ldap_read($ds,"cn=$lost,cn=Users","objectclass=user");
	         		$info=ldap_get_entries($ds,$sr);
                  print "Kicking lost user $lost ";
                  $location=$_SESSION['kicklocn'];
                  $update = array('location' => $location, 'locationautomatic' => '1');
                  $dn="cn=$lost,cn=Users";
                  ldap_modify($ds,$dn,$update);
	      	   	print " from a list of ".count($la2).".<br/>";
                  ldap_close($ds);
 	   	   	}
            }
  	      }
         $_SESSION['lost3']=$_SESSION['lost2'];
         $_SESSION['lost2']=$la;
      }
   }
}

$ds = @ldap_connect("127.0.0.1",4000) or exit("<p>EXIT: LDAP connect failure</p>");
@ldap_bind($ds,"Manager",$ldappasssword) or exit("<p>EXIT: LDAP bind failure</p>");
$attributes=array("cn");
$sr=ldap_list($ds,"cn=Users","(&(objectclass=user)(disabled=1))",$attributes);
$infod=ldap_get_entries($ds,$sr);
if ($infod['count']>0) {
   print "<fieldset style=\"border: 1px solid #000\"><legend style=\"color: #000 \">----- <b>User Disabled</b> (no license?) -----></legend><p>";
   for($n=0; $n<$infod['count'];$n++) {
      print $infod[$n]['cn'][0].", ";
   }
   print "<br/><strong>Total = $n</strong></p></fieldset>";
}

echo "<br/><p>The number after each <b>Registered User</b> is the last octet from their IP address, their hardware is indicated by the bracket type and colour: <b style='color:lime'>[analogue]</b>&nbsp;&nbsp;<b style='color:#808000'>&#60;PCSxxx&#62;</b>&nbsp;&nbsp;<b style='color:red'>{other}</b>&nbsp;&nbsp;<b style='color:#808080'>NotLoggedIn</b></p>";

$sr=ldap_list($ds,"cn=Users","(&(objectclass=user)(virtualuserlicense=1))",$attributes);
$infod=ldap_get_entries($ds,$sr);
if ($infod['count']>0) {
   print "<fieldset style=\"border: 1px solid #000\"><legend style=\"color: #000 \">----- <b>Virtual Users</b> (can't register) -----></legend><p>";
   for($n=0; $n<$infod['count'];$n++) {
      print $infod[$n]['cn'][0].", ";
   }
   print "<br/><strong>Total = $n</strong></p></fieldset></br>";
}
ldap_close($ds);

if ($_SESSION['level'] > 99) {
   foreach ($_SESSION['modules']['ipaddress0'] as $key => $value) {
      $run=time()-$start;
      print "<br/><fieldset style=\"border: 1px solid #000\"><legend style=\"color: #000 \">----- ";
      if ($_SESSION['modules']['live'][$key]) print "<b>";
      print $_SESSION['modules']['cn'][$key];
      if ($_SESSION['modules']['live'][$key]) print "</b>";
      print " -----> ($run Seconds)</legend>";
      if ($run>$runtime) {
         print "<p><b>WARNING: </b>TAKING TOO LONG - CUTTING SHORT <b>:WARNING</b></p>";
         break;
      }
      $locn = $_SESSION['modules']['guid'][$key];
	   if (!$_SESSION['modules']['live'][$key]) $value="127.0.0.1";
      $ds = ldap_connect($value,4000);
	   if ( ! $ds) echo "LDAP connect failure<br>";
	   else {
		   $r = @ldap_bind($ds,"Manager",$ldappasssword);
		   if ( ! $r) {
			   echo "LDAP bind failure<br>";
		   }
/*   temporally disable check for bad locations
          if ($key==0) {
                $attributes=array("cn","location");
    			$sr=@ldap_list($ds,"cn=Users","objectclass=user",$attributes);
	    		$info=ldap_get_entries($ds,$sr);
		    	for($n=0; $n<$info['count'];$n++) {
                    if(!in_array($info[$n]['location'][0],$_SESSION['modules']['guid'])){
                        print $info[$n]['cn'][0]." - unknown location, ";
                    }
                }
            }
*/
   		$attributes=array("cn","locationautomatic","followme","outofoffice","callwaiting");
	   	$sr=@ldap_list($ds,"cn=Users","(&(objectclass=user)(location=$locn))",$attributes);
		   $info=ldap_get_entries($ds,$sr);
			print "<br/>";
         $reg=0;
			for($n=0; $n<$info['count'];$n++) {
            $attributes=array("hardwareid","ipaddress");
   		   $dn="cn=Users.".$info[$n]['cn'][0].",cn=Registrations";
            if (($locn=="00000000-0000-0000-0000-000000000000") or ($_SESSION['modules']['live'][$key]==0)) $sr2=false;
		   	else $sr2=@ldap_read($ds,$dn,"objectclass=RegistrationRecord",$attributes);
			   if(! $sr2) {
               print $info[$n]['cn'][0];
               if (($info[$n]['locationautomatic'][0] == 0) and ($_SESSION['modules']['live'][$key])) {
                    if (!$_SESSION['auto2']) {
                        $update = array( 'locationautomatic' => '1');
 					    ldap_modify($ds,$info[$n]['dn'],$update);
                        print "<b> - automatic set, </b>";
                    } else $_SESSION['auto1']++;
               }
               if($zap and ($_SESSION['modules']['guid'][$key] != '00000000-0000-0000-0000-000000000000') and ($_SESSION['modules']['live'][$key])) {
                  $update = array( 'location' => '00000000-0000-0000-0000-000000000000');
 				      ldap_modify($ds,$info[$n]['dn'],$update);
                  print "<b> clear location </b>";
               }
   			   print ", ";
			   } else {
               $reg++;
				   print "<b>";//.$info[$n]['cn'][0];
               $infoR=ldap_get_entries($ds,$sr2);
               //if (substr($infoR[0]['hardwareid'][0],1,8)=="plicePOT") print "[".substr($infoR[0]['hardwareid'][0],-2)."]";  //show end of mac address
               $i=explode(".",$infoR[0]['ipaddress'][0]);
               if (strpos($infoR[0]['hardwareid'][0],"plicePOT")) print "<span style='color:lime'>{$info[$n]['cn'][0]}[$i[3]]</span>";
               else {
                  if (strpos($infoR[0]['hardwareid'][0],"00-07-d9")) print "<span style='color:#808000'>{$info[$n]['cn'][0]}&lt;$i[3]&gt;</span>";
                  else print "<span style='color:red'>{$info[$n]['cn'][0]}\{$i[3]}</span>";
               }
               print "</b>, ";
			   }
               //if (($info[$n]['followme'][0] < 0) || ($info[$n]['followme'][0] >4)) {
	   		   // print "<b>".$info[$n]['cn'][0]." follow me = ".$info[$n]['followme'][0].", </b>";
			   //}
			   //if (($info[$n]['callwaiting'][0] < 0) || ($info[$n]['callwaiting'][0] >4)) {
			   // print "<b>".$info[$n]['cn'][0]." call waiting = ".$info[$n]['callwaiting'][0].", </b>";
			   //}
			   //if (($info[$n]['outofoffice'][0] < 0) || ($info[$n]['outofoffice'][0] >4)) {
			   // print "<b>".$info[$n]['cn'][0]." out of office = ".$info[$n]['outofoffice'][0].", </b>";
			   //}
		   }
         if ($_SESSION['modules']['live'][$key]==0) $reg="?";
         print "<br/><strong>Users located here = ".$info['count'].", registered here = $reg</strong></fieldset><br/>";
         ldap_close($ds);
		}

   }
}
$run=time()-$start;
print "<p>($run Seconds total to build this page)</p>";

// dont react to locationAutomatic=0 immediately
print "<p>locationAutomatic ".$_SESSION['auto1'].'/'.$_SESSION['auto2']."</p>";
    if ($_SESSION['auto1']){
        $_SESSION['auto1'] =0;
        if($_SESSION['auto2']) $_SESSION['auto2']--;
    } else $_SESSION['auto2']=3;

print date(" H:i:s l, jS F Y");
?>
<br/><br/>
<p><b>Notes:</b></p>
<p>Zero Location is a temporary call server where the users location is not set.<br/>
   Moving users with no location here makes the displaying of registered users easier to see,<br/>
   once the user has registered they will automatically move from Zero location to the call server they have registered to.</p>
</div>
</body>
</html>

